unit Kr_Panel;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ExtCtrls, dsgnIntf;

type
  KrExpandDirection=(edLeft, edRight, edTop, edBottom);
  Kr3d=(dUp, dDown, dNone, dLine);
  KrTitleAlign=(taTop_Left, taCenter, taBottom_Right);
  KrTitleLines=(tlLeft_Top, tlRight_Bottom, tlBoth);
  KrTitleShow=(tsHide, tsNotHide, tsAlways);
  KrAbout=class;
  KrPanel=class(TPanel)
  private
    _HidePanel:TNotifyEvent;
    _UnHidePanel:TNotifyEvent;

    _About:KrAbout;
    _MaxSizeH:integer;
    _MinSizeH:integer;
    _MaxSizeW:integer;
    _MinSizeW:integer;
    _AutoHide:boolean;
    _HideColor:TColor;
    _HideColorUp:TColor;
    _HideColorDown:TColor;
    _HideColorLine:TColor;
    _ExpandDirection:KrExpandDirection;
    _Canvas:TCanvas;
    _HideBevel:Kr3d;
    _Title:string;
    _TitleAlign:KrTitleAlign;
    _TitleLines:KrTitleLines;
    _TitleLineColor:TColor;
    _TitleLineLength:integer;
    _TitleShow:KrTitleShow;

    procedure SetMaxSizeW(value:integer);
    procedure SetMinSizeW(value:integer);
    procedure SetMaxSizeH(value:integer);
    procedure SetMinSizeH(value:integer);
    procedure SetAutoHide(value:boolean);
    procedure SetHideColor(value:TColor);
    procedure SetHideColorUp(value:TColor);
    procedure SetHideColorDown(value:TColor);
    procedure SetHideColorLine(value:TColor);
    procedure SetExpandDirection(value:KrExpandDirection);
    procedure SetHideBevel(value:Kr3d);
    procedure SetTitle(value:string);
    procedure SetTitleAlign(value:KrTitleAlign);
    procedure SetTitleLines(value:KrTitleLines);
    procedure SetTitleLineColor(value:TColor);
    procedure SetTitleLineLength(value:integer);
    procedure SetTitleShow(value:KrTitleShow);

    procedure PanelHide;

    function CalcVertText(t:string):integer;

    procedure CMMouseEnter(var Message:TMessage); message CM_MOUSEENTER;
    procedure CMMouseLeave(var Message:TMessage); message CM_MOUSELEAVE;
    procedure WMPaint(var Message:TWMPaint);      message WM_PAINT;
  protected
    OrgLeft:integer;
    OrgWidth:integer;
    OrgHeight:integer;
    OrgTop:integer;
  public
    property Canvas:TCanvas read _Canvas;

    procedure PaintHideBorder;
    procedure PaintText;

    constructor Create(AOwner:TComponent); override;
    destructor  Destroy; override;

    procedure Loaded; override;
  published
    property About:KrAbout read _About write _About;
    property MaxSizeH:integer read _MaxSizeH write SetMaxSizeH;
    property MinSizeH:integer read _MinSizeH write SetMinSizeH;
    property MaxSizeW:integer read _MaxSizeW write SetMaxSizeW;
    property MinSizeW:integer read _MinSizeW write SetMinSizeW;
    property AutoHide:boolean read _AutoHide write SetAutoHide;
    property HideColor:TColor read _HideColor write SetHideColor;
    property HideColorUp:TColor read _HideColorUp write SetHideColorUp;
    property HideColorDown:TColor read _HideColorDown write SetHideColorDown;
    property HideColorLine:TColor read _HideColorLine write SetHideColorLine;
    property HideBevel:Kr3d read _HideBevel write SetHideBevel;
    property ExpandDirection:KrExpandDirection read _ExpandDirection write SetExpandDirection;
    property Title:string read _Title write SetTitle;
    property TitleAlign:KrTitleAlign read _TitleAlign write SetTitleAlign;
    property TitleLines:KrTitleLines read _TitleLines write SetTitleLines;
    property TitleLineColor:TColor read _TitleLineColor write SetTitleLineColor;
    property TitleLineLength:integer read _TitleLineLength write SetTitleLineLength;
    property TitleShow:KrTitleShow read _TitleShow write SetTitleShow;

    property OnHidePanel:TNotifyEvent read _HidePanel write _HidePanel;
    property OnUnHidePanel:TNotifyEvent read _UnHidePanel write _UnHidePanel;
  end;

  KrAbout=class(TPropertyEditor)
  public
    procedure Edit; override;
    function GetAttributes: TPropertyAttributes; override;
    function GetValue: string; override;
  end;

procedure Register;

implementation

  procedure Register;
  begin
    RegisterComponents('kr...', [KrPanel]);
    RegisterPropertyEditor(TypeInfo(KrAbout), KrPanel, 'About', KrAbout);
  end;

  constructor KrPanel.Create(AOwner:TComponent);
  begin
    inherited;
    _Canvas:=TControlCanvas.Create;
    TControlCanvas(_Canvas).Control:=Self;
  end;

  destructor KrPanel.Destroy;
  begin
    Left:=OrgLeft;
    Width:=OrgWidth;
    Height:=OrgHeight;
    Top:=OrgTop;

    _Canvas.Free;
    inherited;
  end;

  procedure KrPanel.CMMouseEnter(var Message:TMessage);
  begin
    if AutoHide=True then begin
      case ExpandDirection of
        edRight, edBottom: begin
            Left:=OrgLeft;
            Width:=MaxSizeW;
            Top:=OrgTop;
            Height:=MaxSizeH;
          end;
        edLeft: begin
            Left:=(OrgLeft+OrgWidth)-MaxSizeW;
            Width:=MaxSizeW;
            Top:=OrgTop;
            Height:=MaxSizeH;
          end;
        edTop: begin
            Left:=OrgLeft;
            Top:=(OrgTop+OrgHeight)-MaxSizeH;
            Height:=MaxSizeH;
            Width:=MaxSizeW;
          end;
      end;
    end;
    inherited;
    if Assigned(_UnHidePanel) then _HidePanel(Self);
  end;

  procedure KrPanel.CMMouseLeave(var Message:TMessage);
  begin
    inherited;
    if Assigned(_HidePanel) then _HidePanel(Self);
    PanelHide;
  end;

  procedure KrPanel.WMPaint(var Message:TWMPaint);
  begin
    inherited;

    if AutoHide=True then begin
      if not(csDesigning in ComponentState) then begin
        case ExpandDirection of
          edRight: if Width=MinSizeH then PaintHideBorder;
        end;
      end;
    end;
    if Title<>'' then PaintText;
  end;

  procedure KrPanel.PaintText;
  var
    i, h:integer;
    x, y:integer;
    s:integer;
  begin
    SetBkMode(_Canvas.Handle, TRANSPARENT);
    _Canvas.Font.Assign(Font);
    case ExpandDirection of
      edRight, edLeft: begin
          h:=CalcVertText(Title);
          case TitleAlign of
            taTop_Left:      y:=5;
            taCenter:        y:=(Height-h) div 2;
            taBottom_Right:  y:=(Height-h)-5;
          end;

          if ((Width=MinSizeW) and (TitleShow=tsHide)) or
             ((Width<>MinSizeW) and (TitleShow=tsNotHide)) or
             (TitleShow=tsAlways)  then begin
            for i:=1 to length(Title) do begin
              x:=(_Canvas.TextWidth('W')-_Canvas.TextWidth(Copy(Title, i, 1)))div 2;
              if (TitleLines=tlBoth) or (TitleLines=tlLeft_Top) then begin
                _Canvas.Pen.Color:=TitleLineColor;
                s:=4;
                case TitleAlign of
                  taTop_Left:      s:=4;
                  taCenter:        s:=(Height-TitleLineLength) div 2;
                  taBottom_Right:  s:=(Height-TitleLineLength)-4;
                end;
                _Canvas.MoveTo(2, s);
                _Canvas.LineTo(2, s+TitleLineLength);
                x:=x+1;
              end;

              if (TitleLines=tlBoth) or (TitleLines=tlRight_Bottom) then begin
                _Canvas.Pen.Color:=TitleLineColor;
                case TitleAlign of
                  taTop_Left:      s:=4;
                  taCenter:        s:=(Height-TitleLineLength) div 2;
                  taBottom_Right:  s:=(Height-TitleLineLength)-4;
                end;
                _Canvas.MoveTo(5+_Canvas.TextWidth('W'), s);
                _Canvas.LineTo(5+_Canvas.TextWidth('W'), s+TitleLineLength);
                x:=x+3;
              end;
              _Canvas.TextOut(x, y, copy(Title, i, 1));
              y:=y+_Canvas.TextHeight(copy(Title, i, 1));
            end;
          end;
        end;
      edBottom, edTop: begin
          h:=_Canvas.TextWidth(Title);
          case TitleAlign of
            taTop_Left:      x:=5;
            taCenter:        x:=(Width-h) div 2;
            taBottom_Right:  x:=(Width-h)-5;
          end;
          y:=2;
          if (TitleLines=tlBoth) or (TitleLines=tlLeft_Top) then begin
            _Canvas.Pen.Color:=TitleLineColor;
            case TitleAlign of
              taTop_Left:      s:=4;
              taCenter:        s:=(Width-TitleLineLength) div 2;
              taBottom_Right:  s:=(Width-TitleLineLength)-4;
            end;
            _Canvas.MoveTo(s, 3);
            _Canvas.LineTo(s+TitleLineLength, 3);
            y:=y+1;
          end;

          if (TitleLines=tlBoth) or (TitleLines=tlRight_Bottom) then begin
            _Canvas.Pen.Color:=TitleLineColor;
            case TitleAlign of
              taTop_Left:      s:=4;
              taCenter:        s:=(Width-TitleLineLength) div 2;
              taBottom_Right:  s:=(Width-TitleLineLength)-4;
            end;
            _Canvas.MoveTo(s, 6+_Canvas.TextHeight('W'));
            _Canvas.LineTo(s+TitleLineLength, 6+_Canvas.TextHeight('W'));
            y:=y+3;
          end;
          if ((Height=MinSizeH) and (TitleShow=tsHide)) or
             ((Height<>MinSizeH) and (TitleShow=tsNotHide)) or
             (TitleShow=tsAlways)  then begin
            _Canvas.TextOut(x, y, Title);
          end;
        end;
    end;
  end;

  procedure KrPanel.PaintHideBorder;
  begin
    _Canvas.Brush.Color:=_HideColor;
    _Canvas.Brush.Style:=bsSolid;
    _Canvas.FillRect(ClientRect);
    case _HideBevel of
      dUp: begin
          _Canvas.Pen.Color:=_HideColorUp;
          _Canvas.MoveTo(0, Height);
          _Canvas.LineTo(0, 0);
          _Canvas.LineTo(Width-1, 0);
          _Canvas.Pen.Color:=_HideColorDown;
          _Canvas.LineTo(Width-1, Height-1);
          _Canvas.LineTo(0, Height-1);
        end;
      dDown: begin
          _Canvas.Pen.Color:=_HideColorDown;
          _Canvas.MoveTo(0, Height);
          _Canvas.LineTo(0, 0);
          _Canvas.LineTo(Width-1, 0);
          _Canvas.Pen.Color:=_HideColorUp;
          _Canvas.LineTo(Width-1, Height-1);
          _Canvas.LineTo(0, Height-1);
        end;
      dLine: begin
          _Canvas.Brush.Color:=_HideColorLine;
          _Canvas.FrameRect(ClientRect);
        end;
    end;
  end;

  procedure KrPanel.SetMaxSizeW(value:integer);
  begin
    if value=_MaxSizeW then exit;
    _MaxSizeW:=value;
    Invalidate;
  end;

  procedure KrPanel.SetMaxSizeH(value:integer);
  begin
    if value=_MaxSizeH then exit;
    _MaxSizeH:=value;
    Invalidate;
  end;

  procedure KrPanel.SetTitleAlign(value:KrTitleAlign);
  begin
    if value=_TitleAlign then exit;
    _TitleAlign:=value;
    Invalidate;
  end;

  procedure KrPanel.SetTitleLines(value:KrTitleLines);
  begin
    if value=_TitleLines then exit;
    _TitleLines:=value;
    Invalidate;
  end;

  procedure KrPanel.SetTitleLineColor(value:TColor);
  begin
    if value=_TitleLineColor then exit;
    _TitleLineColor:=value;
    Invalidate;
  end;

  procedure KrPanel.SetTitleLineLength(value:integer);
  begin
    if value=_TitleLineLength then exit;
    _TitleLineLength:=value;
    Invalidate;
  end;

  procedure KrPanel.SetMinSizeH(value:integer);
  begin
    if value=_MinSizeH then exit;
    _MinSizeH:=value;
    Invalidate;
  end;

  procedure KrPanel.SetMinSizeW(value:integer);
  begin
    if value=_MinSizeW then exit;
    _MinSizeW:=value;
    Invalidate;
  end;

  procedure KrPanel.SetAutoHide(value:boolean);
  begin
    if value=_AutoHide then exit;
    _AutoHide:=value;
    Invalidate;
  end;

  procedure KrPanel.SetHideColor(value:TColor);
  begin
    if value=_HideColor then exit;
    _HideColor:=value;
    Invalidate;
  end;

  procedure KrPanel.SetHideColorUp(value:TColor);
  begin
    if value=_HideColorUp then exit;
    _HideColorUp:=value;
    Invalidate;
  end;

  procedure KrPanel.SetHideBevel(value:Kr3d);
  begin
    if value=_HideBevel then exit;
    _HideBevel:=value;
    Invalidate;
  end;

  procedure KrPanel.SetTitleShow(value:KrTitleShow);
  begin
    if value=_TitleShow then exit;
    _TitleShow:=value;
    Invalidate;
  end;

  procedure KrPanel.SetHideColorDown(value:TColor);
  begin
    if value=_HideColorDown then exit;
    _HideColorDown:=value;
    Invalidate;
  end;

  procedure KrPanel.SetHideColorLine(value:TColor);
  begin
    if value=_HideColorLine then exit;
    _HideColorLine:=value;
    Invalidate;
  end;

  procedure KrPanel.SetExpandDirection(value:KrExpandDirection);
  begin
    if value=_ExpandDirection then exit;
    _ExpandDirection:=value;
    Invalidate;
  end;

  procedure KrPanel.Loaded;
  begin
    inherited;
    OrgLeft:=Left;
    OrgWidth:=Width;
    OrgHeight:=Height;
    OrgTop:=Top;
    PanelHide;
  end;

  procedure KrPanel.SetTitle(value:string);
  begin
    if value=_Title then exit;
    _Title:=value;
    Invalidate;
  end;

  procedure KrPanel.PanelHide;
  begin
    if AutoHide=True then begin
      Width:=MinSizeW;
      Height:=MinSizeH;
      Left:=OrgLeft;
      Top:=OrgTop;
    end;
  end;

  function KrPanel.CalcVertText(t:string):integer;
  var
    i:integer;
  begin
    result:=0;
    for i:=1 to length(t) do begin
      result:=result+_Canvas.TextHeight(copy(t, i, i));
    end;
  end;

  procedure KrAbout.Edit;
  begin
    MessageDlg('KrPanel'+#13+#13+'Version 1.1, 2000'+#13+
               #13+
               'www.iaehv.nl/users/kray/index.html'+#13+
               'kray@iaehv.nl',
                mtInformation, [mbOK], 0);
  end;

  function KrAbout.GetAttributes: TPropertyAttributes;
  begin
    Result := [paMultiSelect, paDialog, paReadOnly];
  end;

  function KrAbout.GetValue: string;
  begin
    Result := '(about)';
  end;
end.
